import os.path

from .base import Model, Collection, DATA_DIRECTORY

NEIGHBORHOOD_CSV_FILENAME = os.path.join(DATA_DIRECTORY,
    'Neighborhoods_2012b.csv')

class Neighborhood(Model):
    fields = [
        'name',
    ]

    def __str__(self):
        return self.name

    def __repr__(self):
        return "Neighborhood(name='{ca.name}')".format(
            ca=self)


class NeighborhoodCollection(Collection):
    model = Neighborhood

    def transform_row(self, row):
        return {
            'name': row['PRI_NEIGH'],
        }

    def default_sort(self):
        self._items = sorted(self._items, key=lambda n: n.name)
        return self



NEIGHBORHOODS = NeighborhoodCollection().from_csv(NEIGHBORHOOD_CSV_FILENAME)



import os.path

from .base import Model, Collection, DATA_DIRECTORY

COMMUNITY_AREA_CSV_FILENAME = os.path.join(DATA_DIRECTORY, 'CommAreas.csv')

class CommunityArea(Model):
    fields = [
        'name',
        'number',
    ]

    def __str__(self):
        return self.name

    def __repr__(self):
        return "CommunityArea(name='{ca.name}', number='{ca.number}')".format(
            ca=self)


class CommunityAreaCollection(Collection):
    model = CommunityArea

    def __init__(self):
        self._by_number = {}
        super(CommunityAreaCollection, self).__init__()

    def add_item(self, item):
        super(CommunityAreaCollection, self).add_item(item)
        self._by_number[item.number] = item

    def transform_row(self, row):
        return {
            'name': row['COMMUNITY'].title(),
            'number': row['AREA_NUMBE'],
        }

    def get_by_number(self, number):
        number_s = str(number)
        return self._by_number[number_s]

    def default_sort(self):
        self._items = sorted(self._items, key=lambda ca: int(ca.number))
        return self


COMMUNITY_AREAS = CommunityAreaCollection().from_csv(COMMUNITY_AREA_CSV_FILENAME)



from setuptools import find_packages, setup

# PyPI only supports nicely-formatted README files in reStructuredText.
# Newsapps seems to prefer Markdown.  Use a version of the pattern from
# https://coderwall.com/p/qawuyq/use-markdown-readme-s-in-python-modules
# to convert the Markdown README to rst if the pypandoc package is
# present.
try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError, OSError):
    long_description = open('README.md').read()

# Load the version from the version module
exec(open('chicago/version.py').read())

setup(
    name='chicago',
    version=__version__,
    author='Geoff Hing for the Chicago Tribune News Applications Team',
    author_email='geoffhing@gmail.com',
    url='https://github.com/newsapps/python-chicago',
    description='Chicago meta information and other fun stuff',
    packages=find_packages(exclude=["tests", "tests.*"]),
    include_package_data=True,
    install_requires=[
    ],
    tests_require=[
        'nose',
    ],
    test_suite='nose.collector',
    keywords=['chicago', 'geography', 'city'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



import csv
import os.path


DATA_DIRECTORY = os.path.join(os.path.dirname(os.path.abspath(__file__)),
    'data')


class Model(object):
    fields = []

    def __init__(self, **kwargs):
        for field in self.fields:
            try:
                val = kwargs[field]
                setattr(self, field, val)
            except KeyError:
                pass


class Collection(object):
    model = Model

    def __init__(self, items=None):
        self._items = []
        if items is None:
            items = []
        for item in items:
            self.add_item(item)

        self.default_sort()

    def __iter__(self):
        return iter(self._items);

    def __getitem__(self, i):
        return self._items[i]

    def __len__(self):
        return len(self._items)

    def __repr__(self):
        cls_name = self.__class__.__name__
        return "{}([{}])".format(
            cls_name,
            ','.join([repr(item) for item in self._items])
        )

    def default_sort(self):
        return self

    def get_model(self):
        return self.model

    def add_item(self, item):
        self._items.append(item)

    def _from_csv_file(self, csvfile):
        reader = csv.DictReader(csvfile)
        model_cls = self.get_model()
        for row in reader:
            model_kwargs = self.transform_row(row)
            self.add_item(model_cls(**model_kwargs))

        self.default_sort()

        return self

    def from_csv(self, csvfile):
        try:
            return self._from_csv_file(csvfile)
        except KeyError:
            # Couldn't parse CSV properly, so csvfile is probably a string
            with open(csvfile, 'r') as f:
                return self._from_csv_file(f)

    def transform_row(self, row):
        return row
